import requests
from bs4 import BeautifulSoup

from .util import SchedulerError


def scrap_term_dates(year, term):
    url = f"https://calendar.ucf.edu/{year}/{term}"
    try:
        r = requests.get(url)
    except requests.ConnectionError as e:
        raise SchedulerError("There's a problem with your internet connection. Please, try again.") from e
    soup = BeautifulSoup(r.text, "html.parser")
    titles = soup.find_all("h2", {"class": "mt-3 mb-2"})
    for title in titles:
        if title.get_text().startswith("Academic Dates and Deadlines"):
            deadline_table = title.find_next_sibling("table")
    for elem in deadline_table.find_all("tr"):
        summary = elem.find("span", {"class": "summary"})
        if summary is not None:
            if summary.get_text().startswith("Classes Begin"):
                start_date = elem.find("abbr", {"class": "dtstart"})['title']
            elif summary.get_text().startswith("Classes End"):
                end_date = elem.find("abbr", {"class": "dtstart"})['title']
    return start_date, end_date


def scrap_no_school_events(year, term):
    url = f"https://calendar.ucf.edu/{year}/{term}/no-classes/"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "html.parser")
    raw_events = soup.find_all("tr", {"class": "vevent"})
    scrapped_events = []
    for event in raw_events:
        dtstart = dtend = description = None
        for elem in event.find_all("abbr"):
            class_ = elem['class'] if isinstance(
                elem['class'], str) else elem['class'][0]
            if class_ == "dtstart":
                dtstart = elem['title']
            elif class_ == "dtend":
                dtend = elem['title']
        raw_description = event.find("div", {"class": "more-details"})
        if raw_description is not None:
            description = raw_description.get_text().strip()
        if dtstart is None:
            # Sometimes it has an event with no dtstart and no dtend.
            # I would check back on it later (UCF Cal -> no-school tag -> Study day)
            continue
        scrapped_events.append(dict(
            summary=event.find("span", {"class": "summary"}).get_text(),
            raw_dtstart=dtstart,
            raw_dtend=dtend
        ))
        if description:
            scrapped_events[-1]['description'] = description
    return scrapped_events



from setuptools import setup


requires = [
    'icalendar',
    'bs4',
    'requests'
]

setup(
    name="unischeduler",
    version="0.1.3",
    packages=['unischeduler'],
    install_requires=requires,

    # metadata to display on PyPI
    author="Stanislav Zmiev",
    author_email="szmiev2000@gmail.com",
    description="Converts class schedule from text to an icalendar format to export into any known calendar service (google calendar, apple calendar, outlook calendar, etc)",
    license="MIT",
    project_urls={"Source Code": "https://github.com/Varabe/unischeduler"},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ]
)



import re
from typing import List, Tuple, Union


def parse_schedule(schedule: str) -> List[Union[str, List[str]]]:
    schedule = "\n".join([l.strip()
                          for l in schedule.splitlines() if l.strip()])
    re_summary = re.compile(r"[A-Z]{3}[A-Z]* \d+[A-Z]? - .+")
    # Good luck figuring this out!
    re_details = re.compile(
        r"(?:[A-Z][a-z]+\n)?(?:[A-Z][a-z])+ \d\d?:\d\d.+\n.+(?:[A-Za-z:\-\s\d,\.])+.+")
    class_summaries = re_summary.findall(schedule)
    classes = re_summary.split(schedule)[1:]  # classes[0] == ''
    sections: List[Union[List[str], str]] = []
    for class_summary, class_info in zip(class_summaries, classes):
        if "Dropped" in class_info or "Online" in class_info or "Withdrawn" in class_info:
            continue
        current_sections = re_details.findall(class_info)
        for section in current_sections:
            lines = [l.strip() for l in section.splitlines() if l.strip()]
            if " - " not in lines[0]:  # If it's not a recurrence line
                section_type = last_type = lines.pop(0)
            else:
                section_type = last_type
            recurrence, location, *professors, dtstart_and_dtend = lines
            professors = [p.replace(",", "") for p in professors]
            start_time, end_time, weekdays = extract_time_attributes(
                recurrence)
            dtstart, _, dtend = dtstart_and_dtend.split(" ")
            sections.append([class_summary, section_type, weekdays,
                             start_time, end_time, location, professors, dtstart, dtend])
    return sections


# MoWeFr 9:30AM - 10:20AM
def extract_time_attributes(time_line: str) -> Tuple[str, str, List[str]]:
    weekdays_str, start_time, _, end_time = time_line.split(" ")
    weekdays = [weekdays_str[i:i + 2] for i in range(0, len(weekdays_str), 2)]
    return start_time, end_time, weekdays
